<?php

App::uses('File', 'Utility');

class ConnecteursController extends AppController {

    const NS_CMIS_RA = "http://docs.oasis-open.org/ns/cmis/restatom/200908/";
    const NS_CMIS = "http://docs.oasis-open.org/ns/cmis/core/200908/";

    /**
     * Controller name
     *
     * @var string
     * @access public
     */
    public $name = 'Connecteurs';
    public $components = array('Iparapheur', 'Cmis', 'Localeo');

    /**
     * Gestion des connecteurs interface graphique)
     *
     * @logical-group Connecteurs
     * @user-profil Admin
     *
     * @access public
     * @return void
     */
    public function index() {
        $this->set('ariane', array(
            '<a href="/environnement/index/0/admin">' . __d('menu', 'Administration', true) . '</a>',
            __d('menu', 'gestionConnecteurs', true)
        ));
    }

    
    /**
     *
     */
    public function beforeFilter() {
        parent::beforeFilter();
        $this->Auth->allow('sync');
    }
    /**
     * Récupération de la liste des connecteurs
     *
     * @user-profile Admin
     *
     * @access public
     * @return void
     */
    public function getConnecteurs() {

        $connecteurs_tmp = $this->Connecteur->find("all", array('order' => 'Connecteur.name'));
        $connecteurs = array();
        foreach ($connecteurs_tmp as $item) {
            if ($item['Connecteur']['name'] == 'GED (CMIS)') {
                $item['url'] = Configure::read('BaseUrl') . '/connecteurs/getGed';
                $isGed = true;
            } else {
                $item['url'] = Configure::read('BaseUrl') . '/connecteurs/getCircuitsParapheur';
                $isGed = false;
            }

            $item['right_edit'] = true;
            $item['right_check'] = true;
            $item['right_sync'] = false;
            if ($item['Connecteur']['name'] == 'Annuaire LDAP') {
                $item['right_sync'] = true;
            }
            $item['right_delete'] = true;
            $connecteurs[] = $item;
        }

        $this->set('isGed', $isGed);
        $this->set('connecteurs', $connecteurs);
        
        $conn = $this->Session->read('Auth.User.connName');
        $this->loadModel('Collectivite');
        $this->set('collectivite', $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $conn))));
    }

    public function edit($id) {
        $type = array(
            'ActiveDirectory' => 'Active Directory',
            'OpenLDAP' => 'Open LDAP'
        );
        $this->set( 'type', $type);
        
        $this->request->data = $this->Connecteur->find(
                'first', array(
            'conditions' => array(
                'Connecteur.id' => $id
            ),
            'contain' => false
                )
        );

        // Configuration signature
        $protocoles = array('PASTELL' => 'Pastell', 'IPARAPHEUR' => 'i-Parapheur');
        $visibilite = array(
            'PUBLIC' => 'PUBLIC',
            'CONFIDENTIEL' => 'CONFIDENTIEL',
            'SERVICE' => 'GROUPE'
        );
        if (!Configure::read('USE_PASTELL')) {
            unset($protocoles['PASTELL']);
        }
        $this->set('protocoles', $protocoles);
        $this->set('visibilite', $visibilite);

        if (stristr($this->request->data['Connecteur']['name'], 'Parapheur')) {
            $this->render('signature');
        } else if (stristr($this->request->data['Connecteur']['name'], 'GED')) {
            $this->render('cmis');
        } else if (stristr($this->request->data['Connecteur']['name'], 'LDAP')) {
            $this->render('ldap');
        } else if (stristr($this->request->data['Connecteur']['name'], 'GRC')) {
            $this->render('grc');
        } else {
            $this->Session->setFlash('Ce connecteur n\'est pas valide', 'growl', array('type' => 'erreur'));
            return $this->redirect(array('action' => 'index'));
        }
    }

    public function _replaceValue($content, $param, $new_value) {
        if (is_bool(Configure::read($param)))
            $valeur = Configure::read($param) === true ? 'true' : 'false';
        else
            $valeur = Configure::read($param);

        $host_b = "Configure::write('$param', '" . $valeur . "');";
        $host_a = "Configure::write('$param', '$new_value');";
        $return = str_replace($host_b, $host_a, $content, $count);
        if ($count === 0) {
            $host_b = "Configure::write('$param', " . $valeur . ");";
            $host_a = "Configure::write('$param', $new_value);";

            $return = str_replace($host_b, $host_a, $content, $count);
        }

        return $return;
    }

    public function makeconf($type) {
//debug($this->params);
//die();
        $this->autoRender = false;
        $this->Jsonmsg->init();
//        $file = new File(APP . 'Config' . DS . 'webgfc.inc', true);
//        $content = $file->read();
        switch ($type) {
            case 'signature' :
                $this->Jsonmsg->init();
                if (!empty($this->request->data)) {
                    $this->Connecteur->begin();
                    $this->Connecteur->create();
//
                    if (!empty($this->request->params['form']['myfile']['tmp_name'])) {
                        $certs = array();
                        $this->request->data['Connecteur']['nom_cert'] = $this->request->params['form']['myfile']['name'];
                        $pkcs12 = file_get_contents($this->request->params['form']['myfile']['tmp_name']);
                        if (openssl_pkcs12_read($pkcs12, $certs, $this->data['Connecteur']['certpwd'])) {
                            $this->request->data['Connecteur']['cacert'] = $certs['pkey'] . $certs['cert'];
                            $this->request->data['Connecteur']['clientcert'] = $certs['extracerts'][0];
                        } else
                            $this->Jsonmsg->error('Le mot de passe du certificat est erroné');
                    }
                    else {
                        unset($this->request->params['form']['myfile']);
                    }
                    
                    if ($this->Connecteur->save($this->request->data)) {
                        $this->Connecteur->commit();
                        $this->Jsonmsg->valid();
                    } else {
                        $this->Jsonmsg->error('Erreur lors de l\'enregistrement');
                        $this->Connecteur->rollback();
                    }
                    $this->Jsonmsg->send();
                }
                break;
            case 'cmis' :
                $this->Jsonmsg->init();
                if (!empty($this->request->data)) {
                    $this->Connecteur->begin();
                    $this->Connecteur->create();
                    if ($this->Connecteur->save($this->request->data)) {
                        $this->Connecteur->commit();
                        $this->Jsonmsg->valid();
                    } else {
                        $this->Jsonmsg->error('Erreur lors de l\'enregistrement');
                        $this->Connecteur->rollback();
                    }
                    $this->Jsonmsg->send();
                }
                break;
            case 'ldap' :
                $this->Jsonmsg->init();
                if (!empty($this->request->data)) {
                    $this->Connecteur->begin();
                    $this->Connecteur->create();
                    if ($this->Connecteur->save($this->request->data)) {
                        $this->Connecteur->commit();
                        $this->Jsonmsg->valid();
                    } else {
                        $this->Jsonmsg->error('Erreur lors de l\'enregistrement');
                        $this->Connecteur->rollback();
                    }
                    $this->Jsonmsg->send();
                }
                break;
            case 'grc' :
                $this->Jsonmsg->init();
                if (!empty($this->request->data)) {
                    $this->Connecteur->begin();
                    $this->Connecteur->create();
                    if ($this->Connecteur->save($this->request->data)) {
                        $this->Connecteur->commit();
                        $this->Jsonmsg->valid();
                    } else {
                        $this->Jsonmsg->error('Erreur lors de l\'enregistrement');
                        $this->Connecteur->rollback();
                    }
                    $this->Jsonmsg->send();
                }
                break;
            case 'debug' :
                $content = $this->_replaceValue($content, 'debug', $this->data['Connecteur']['debug']);
                break;
            case 'all' :
                $content = $this->data['Connecteur']['all'];
                break;
            default :
                $this->Jsonmsg->error('Ce connecteur n\'est pas valide');
//                return $this->redirect(array('action' => 'index'));
        }

//        $this->Jsonmsg->send();
// debug($this->request->data);
// die();
        return $this->redirect(array('action' => 'index'));
    }

    public function getCircuitsParapheur() {
        $this->Jsonmsg->init();
        $time_start = microtime(true);
        $circuits = '';
        if (!Configure::read('USE_PARAPHEUR')) {
            $this->Session->setFlash('Utilisation du parapheur désactivé', 'growl', array('type' => 'info'));
            $this->Jsonmsg->send();
            return;
        }
        if (Configure::read('PARAPHEUR') == 'IPARAPHEUR') {
            $parapheur = $this->Connecteur->find(
                    'first', array(
                'conditions' => array(
                    'Connecteur.name ILIKE' => '%Parapheur%'
                ),
                'contain' => false
                    )
            );
            $Parafwebservice = new IparapheurComponent();
            $circuits = $Parafwebservice->getListeSousTypesWebservice($parapheur['Connecteur']['type'], $parapheur);

            if ($circuits === 'Connexion impossible') {
                $this->Jsonmsg->error("Impossible de se connecter à l'hôte");
                $this->Jsonmsg->send();
                return;
            }
            $time_end = microtime(true);
            if (!empty($circuits["soustype"])) {
//                $hostparapheur = "Parapheur contacté : ". Configure::read('IPARAPHEUR_HOST');
                $hostparapheur = "Parapheur contacté : " . $parapheur['Connecteur']['host'];
                $time = round($time_end - $time_start, 2);
                $nbCircuits = "Réponse du Parapheur : " . count($circuits['soustype']) . " circuit(s) du i-parapheur récupéré(s) en $time secondes";

                $messageSuccess = $hostparapheur . "<br />" . $nbCircuits;
                $this->Jsonmsg->valid($messageSuccess);
            } else
                $this->Jsonmsg->error('La liste de circuit du i-parapheur est vide');
            $this->Jsonmsg->send();
        }
    }


    public function check($id) {
        $this->request->data = $this->Connecteur->find(
            'first',
            array(
                'conditions' => array(
                    'Connecteur.id' => $id
                ),
                'contain' => false
            )
        );

        if (stristr($this->request->data['Connecteur']['name'], 'Parapheur')) {
            $parapheur = $this->request->data;

            // Configuration signature
            $this->Jsonmsg->init();
            $time_start = microtime(true);
            $circuits = '';
            if (!$parapheur['Connecteur']['use_signature']) {
                $this->Jsonmsg->valid("Utilisation du parapheur désactivé");
                $this->Jsonmsg->send();
                return;
            }

            $Parafwebservice = new IparapheurComponent();
            $circuits = $Parafwebservice->getListeSousTypesWebservice($parapheur['Connecteur']['type'], $parapheur);

            if ($circuits === 'Connexion impossible') {
                $this->Jsonmsg->error("Impossible de se connecter à l'hôte");
                $this->Jsonmsg->send();
                return;
            }
            $time_end = microtime(true);
            if (!empty($circuits["soustype"])) {
                $hostparapheur = "Parapheur contacté : " . $parapheur['Connecteur']['host'];
                $time = round($time_end - $time_start, 2);
                $nbCircuits = "Réponse du Parapheur : " . count($circuits['soustype']) . " circuit(s) du i-parapheur récupéré(s) en $time secondes";

                $messageSuccess = $hostparapheur . "<br />" . $nbCircuits;
                $this->Jsonmsg->valid($messageSuccess);
            } else
                $this->Jsonmsg->error('La liste de circuit du i-parapheur est vide');
            $this->Jsonmsg->send();
//            }
        }
        else if (stristr($this->request->data['Connecteur']['name'], 'GED')) {
            // Connecteur CMIS/GED
            $this->Jsonmsg->init();
            $time_start = microtime(true);

            // Le connecteur est-il actif
            if (!$this->request->data['Connecteur']['use_ged']) {
                $this->Jsonmsg->error('Utilisation de la GED désactivée', 'growl', array('type' => 'info'));
                $this->Jsonmsg->send();
                return;
            }

            require_once( ROOT . DS . APP_DIR . DS . 'Lib' . DS . 'cmis.php' );

            // Le couple login/mdp est-il le bon
            $configured = Cmis::configured();
            if (!$configured) {
                $this->Jsonmsg->error("La connexion avec la GED a échoué : couple login/mot de passe erroné.", 'growl', array('type' => 'info'));
                $this->Jsonmsg->send();
                return;
            }


            // Existence ou non du répertoire renseigné
            if ($this->request->data['Connecteur']['ged_cmis_version'] == '1.1') {
                $cmis = new CmisComponent;
                $cmis->CmisComponent_Service();

                $folderUsed = $cmis->client->getObjectByPath($this->request->data['Connecteur']['ged_repo']);

                if ($folderUsed == 'emptyRepo') {
                    $this->Jsonmsg->error("La connexion avec la GED a échoué : le dossier " . $this->request->data['Connecteur']['ged_repo'] . " n'a pas été trouvé</ul>", 'growl', array('type' => 'info'));
                    $this->Jsonmsg->send();
                    return;
                } else {
                    $messageSuccess = "La connexion est réussie - web-GFC a récupéré les informations suivantes :<ul>";
                    foreach ($this->getFolderRetrieveInfo11() as $properties) {
                        $messageSuccess .= "<li> $properties  : " . $folderUsed->properties['cmis:' . $properties] . "</li>";
                    }
                    $messageSuccess .="</ul>";
                }

                $this->Jsonmsg->valid($messageSuccess);
                $this->Jsonmsg->send();
            } else {
                $folderUsed = $this->testObject($this->request->data['Connecteur']['ged_repo']);
                if ($folderUsed === false) {
                    $this->Jsonmsg->error("La connexion avec la GED a échoué : le dossier " . $this->request->data['Connecteur']['ged_repo'] . " n'a pas été trouvé</ul>", 'growl', array('type' => 'info'));
                    $this->Jsonmsg->send();
                    return;
                }


                // si tout ok
                $messageSuccess = "La connexion est réussie - web-GFC a récupéré les informations suivantes :<ul>";
                foreach ($this->getFolderRetrieveInfo() as $repoInfo) {
                    $messageSuccess .= "<li> $repoInfo  : " . $folderUsed[$repoInfo] . "</li>";
                }
                $messageSuccess .="</ul>";


                $info = $this->getRepositoryInfo();

                $this->Jsonmsg->valid($messageSuccess);
                $this->Jsonmsg->send();
            }
        }
        else if (stristr($this->request->data['Connecteur']['name'], 'Annuaire LDAP')) {
            $this->loadModel('User');
             // Connecteur CMIS/GED
            $this->Jsonmsg->init();
            $time_start = microtime(true);
            $ldap = $this->request->data;
            // Le connecteur est-il actif
            if (!$ldap['Connecteur']['use_ldap']) {
                $this->Jsonmsg->error('Utilisation du LDAP désactivé', 'growl', array('type' => 'info'));
                $this->Jsonmsg->send();
                return;
            }

            // On véreifie la connexion à l'annuaire LDAP défini
            $conn = ldap_connect($ldap['Connecteur']['ldap_host'], $ldap['Connecteur']['ldap_port']) or die("connexion impossible au serveur LDAP");
            ldap_set_option($conn, LDAP_OPT_PROTOCOL_VERSION, 3);
            ldap_set_option($conn, LDAP_OPT_REFERRALS, 0); // required for AD
            $bind = @ldap_bind($conn, $ldap['Connecteur']['ldap_login'], $ldap['Connecteur']['ldap_password']);
            if( $bind ) {
                $message = "La connexion à l'annuaire LDAP a réussi";
                $this->Jsonmsg->valid($message);
            }
            else {
                $message = "Echec de la connexion à l'annuaire LDAP";
                $this->Jsonmsg->error( $message);
            }
            $this->Jsonmsg->send();
        }
        else if (stristr($this->request->data['Connecteur']['name'], 'GRC')) {
//            $this->autoRender = false;
            $this->Jsonmsg->init();
//            $time_start = microtime(true);
            $grc = $this->request->data;
            // Le connecteur est-il actif ?
            if (!$grc['Connecteur']['use_grc']) {
                $this->Jsonmsg->error('Utilisation de la GRC désactivée', 'growl', array('type' => 'info'));
                $this->Jsonmsg->send();
                return;
            }

            // On vérifie la connexion à la GRC
            $GRCwebservice = new LocaleoComponent();
            $echoTest = $GRCwebservice->echoTest();
            if( !empty($echoTest) ) {
                if(isset( $echoTest->err_msg)) {
                    $message = "La connexion à la GRC a réussi. <br /> retour de la GRC : ".$echoTest->err_msg;
                }
                else {
                    $message = "La connexion à la GRC a réussi. <br /> Retour de la GRC : ".$echoTest->statut->label;
                }
                $this->Jsonmsg->valid($message);
            }
            else {
                $message = "Echec de la connexion à la GRC";
                $this->Jsonmsg->error($message);
            }
            $this->Jsonmsg->send();
        }
        else {
            $this->Session->setFlash('Ce connecteur n\'est pas valide', 'growl', array('type' => 'erreur'));
            return $this->redirect(array('action' => 'index'));
        }
    }

    public function getRepositoryInfo() {
//		$xmldata = $this->get(Configure::read('CMIS_HOST'));
        $cmis = $this->Connecteur->find(
                'first', array(
            'conditions' => array(
                'Connecteur.name ILIKE' => '%GED%'
            ),
            'contain' => false
                )
        );
        $xmldata = $this->get($cmis['Connecteur']['ged_url']);

        if (!$xmldata) {
            return false;
        }

        libxml_use_internal_errors(true);
        $xml = simplexml_load_string($xmldata);

        if (!$xml) {
            $errors = libxml_get_errors();
            $this->lastError = "Erreur XML : " . $errors[0]->message;
            return false;
        }

        $repInfo = $xml->workspace->children(self::NS_CMIS_RA)->repositoryInfo;

        $result = array();
        foreach ($this->getRepositoryRetrieveInfo() as $infoName) {
            $result[$infoName] = strval($repInfo->children(self::NS_CMIS)->$infoName);
        }

        $uriTemplate = $xml->workspace->children(self::NS_CMIS_RA)->uritemplate;

        foreach ($uriTemplate as $template) {
            $type = strval($template->children(self::NS_CMIS_RA)->type);
            $result['template'][$type] = strval($template->children(self::NS_CMIS_RA)->template);
        }

        return $result;
    }

    private function get($url, $content = false) {
        $cmis = $this->Connecteur->find(
                'first', array(
            'conditions' => array(
                'Connecteur.name ILIKE' => '%GED%'
            ),
            'contain' => false
                )
        );
        $session = curl_init($url);

        curl_setopt($session, CURLOPT_HEADER, false);
        curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
//        curl_setopt($session, CURLOPT_USERPWD, Configure::read('CMIS_LOGIN') . ":" . Configure::read('CMIS_PWD'));
        curl_setopt($session, CURLOPT_USERPWD, $cmis['Connecteur']['ged_login'] . ":" . $cmis['Connecteur']['ged_passwd']);

        if ($content) {
            curl_setopt($session, CURLOPT_CUSTOMREQUEST, "POST");
            curl_setopt($session, CURLOPT_POSTFIELDS, $content);
            curl_setopt($session, CURLOPT_HTTPHEADER, array("Content-Type: application/atom+xml;type=entry"));
        } else {
            curl_setopt($session, CURLOPT_CUSTOMREQUEST, "GET");
        }
        $result = curl_exec($session);
        $codeResponse = curl_getinfo($session, CURLINFO_HTTP_CODE);

        if (!in_array($codeResponse, array('200', '201'))) {
            $this->lastError = curl_error($session);
            if (!$this->lastError) {
                $this->lastError = "Erreur $codeResponse (la GED a retourné : $result)";
            }
            return false;
        }

        return $result;
    }

    public function getRepositoryRetrieveInfo() {
        return array('repositoryId', 'repositoryName', 'repositoryDescription', 'vendorName', 'productName', 'productVersion', 'rootFolderId');
    }

    public function getRootFolder() {
        $cmis = $this->Connecteur->find(
                'first', array(
            'conditions' => array(
                'Connecteur.name ILIKE' => '%GED%'
            ),
            'contain' => false
                )
        );
//		return Configure::read('CMIS_REPO');
        return $cmis['Connecteur']['ged_url'];
    }

    public function testObject($repoPath) {
        return $this->getObjectByPath($repoPath);
    }

    public function getObjectByPath($path) {

        $repositoryInfo = $this->getRepositoryInfo();

        if (!$repositoryInfo) {
            return false;
        }

        $url_template = $repositoryInfo['template']['objectbypath'];

        $url = str_replace("{path}", $path, $url_template);
        $url = preg_replace("/{[a-zA-Z0-9_]+}/", "", $url);

        $xmldata = $this->get($url);

        if (!$xmldata) {
            return false;
        }

        libxml_use_internal_errors(true);
        $xml = simplexml_load_string($xmldata);
        if (!$xml) {
            $errors = libxml_get_errors();
            $this->lastError = "Erreur XML : " . $errors[0]->message;
            return false;
        }


        $result = array('author' => strval($xml->author->name));
        foreach ($this->getFolderRetrieveInfo() as $infoName) {
            $result[$infoName] = strval($xml->$infoName);
        }

        foreach ($xml->link as $link) {
            if (!isset($result['link'][strval($link['rel'])])) {
                $result['link'][strval($link['rel'])] = strval($link['href']);
            }
        }

        return $result;
    }

    public function getFolderRetrieveInfo() {
        return array('content', 'id', 'summary', 'title', 'published', 'updated');
    }

    /**
     * Fonction de suppression d'un connecteur
     * @param type $id
     */
    public function delete($id) {
        $this->Jsonmsg->init(__d('default', 'delete.error'));
        if ($this->Connecteur->delete($id)) {
            $this->Jsonmsg->valid(__d('default', 'delete.ok'));
        }
        $this->Jsonmsg->send();
    }

    /**
     * Fonction remontant les infos en version 1.1 de CMIS
     * @return type
     */
    public function getFolderRetrieveInfo11() {
        return array('path', 'lastModifiedBy', 'name', 'creationDate', 'description', 'createdBy');
    }


    /**
     * Fonctin permettant la synchronisation LDAP manuelle
     * @param type $id
     * @return type
     */
    public function sync($id) {
        $this->request->data = $this->Connecteur->find(
            'first',
            array(
                'conditions' => array(
                    'Connecteur.id' => $id
                ),
                'contain' => false
            )
        );
        if (stristr($this->request->data['Connecteur']['name'], 'Annuaire LDAP')) {
            $this->loadModel('User');
            $this->Jsonmsg->init();
            $time_start = microtime(true);
            $ldap = $this->request->data;
            
            // Le connecteur est-il actif
            if (!$ldap['Connecteur']['use_ldap']) {
                $this->Jsonmsg->error('Utilisation du LDAP désactivé', 'growl', array('type' => 'info'));
                $this->Jsonmsg->send();
                return;
            }

            // On vérifie la connexion à l'annuaire LDAP défini  
            // si tout ok
            $messageSuccess = "La connexion est réussie - web-GFC a récupéré les informations suivantes :<br /><ul>";
            $cmdUser = 'sudo -u www-data '. APP . DS . 'Console/cake ldap_manager user_update -c '.Configure::read('conn');
            $commandShell = shell_exec($cmdUser);
            $userIsCreated = strpos( $commandShell, 'a été créé'  );
            $userAlreadyExists = strpos( $commandShell, 'existe déjà'  );            
            if( Configure::read('debug') == 2 ) {
                if(  $userIsCreated !== false ) {
                    $messageSuccess .=  substr($commandShell, 0, strpos($commandShell, '1. search'));
                    $messageSuccess = str_replace( " a été créé", " a été créé " , nl2br( $messageSuccess ) );
                }
                else if(  $userAlreadyExists !== false ) {
                    $messageSuccess .=  substr($commandShell, 0, strpos($commandShell, '1. search'));
                    $messageSuccess = str_replace( " existe déjà", " existe déjà" , nl2br( $messageSuccess ) );
                }
            }
            else {
                $messageSuccess .= $commandShell;
            }

            $this->set( compact( 'messageSuccess' ) );

//            $this->Jsonmsg->send();
            $this->render('sync');
        }
        else {
            $this->Session->setFlash('Ce connecteur n\'est pas valide', 'growl', array('type' => 'erreur'));
            return $this->redirect(array('action' => 'index'));
        }
    }
}
