<?php

/**
 * Recherches
 *
 * Recherches controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Stéphane Sampaio
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class RecherchesController extends AppController {

    /**
     * Controller name
     *
     * @var string
     * @access public
     */
    public $name = 'Recherches';

    /**
     * Controller uses
     *
     * @var array
     * @access public
     */
    public $uses = array('Recherche', 'Courrier', 'Ordreservice', 'Pliconsultatif', 'Consultation');

    /**
     * Controller components
     *
     * @var array
     * @access public
     */
    public $components = array('DataAuthorized');

    /**
     * Controller helpers
     *
     * @var array
     * @access public
     */
    public $helpers = array('Csv', 'Paginator');

    //TODO: enlever le beforeFilter et régénérer les droits
    public function beforeFilter() {
        $this->Auth->allow(array('index', 'exportcsv', 'export')); //FIXME
        parent::beforeFilter();
    }

    protected function _setOptions() {
        $this->loadModel('Desktop');
        $desktopId = $this->Session->read('Auth.User.Courrier.Desktop.id');
        $daros = $this->Desktop->getDaros($desktopId);
        $this->set('taches', $this->Recherche->Tache->find('list', array('order' => array('Tache.name ASC'))));
        $this->set('circuits', $this->Recherche->Circuit->find('list', array('order' => array('Circuit.nom ASC'))));
        $this->set('dossiers', $this->Recherche->Dossier->find('list', array('order' => array('Dossier.name ASC'))));
        $this->set('affaires', $this->Recherche->Affaire->find('list', array('order' => array('Affaire.name ASC'))));
        $this->set('origineflux', $this->Courrier->Origineflux->find('list', array('order' => array('Origineflux.name ASC'))));
        $metas = $this->Recherche->Metadonnee->find('all', array('contain' => array('Selectvaluemetadonnee', 'Typemetadonnee')));
        $this->set('metas', $metas);
        $valueSelectMeta = array('Non' => 'Non', 'Oui' => 'Oui');
        $this->set('valueSelectMeta', $valueSelectMeta);
        $contacts = $this->Recherche->Contact->find('list', array('order' => 'Contact.name ASC'));
        $this->set('contacts', $contacts);
        if( Configure::read('CD') != 81) {
            $organismes = $this->Recherche->Organisme->find('list', array('order' => 'Organisme.name ASC'));
            $this->set('organismes', $organismes);
        }
        else {
            $organsme = $this->Recherche->Organisme->find('list', array( 'fields' => array('Organisme.name', 'Organisme.ville',  'Organisme.id'), 'order' => array('Organisme.name ASC')));
            $organismes = array();
            foreach( $organsme as $id => $value ) { 
                $org = implode( ",", array_keys( $value )  );
                $ville = implode( ",", $value );
                $organismes[$id] = $org.' ( '.$ville . ' )';
            }
            $this->set('organismes', $organismes );

        }

        if (Configure::read('Use.AuthRights') && !Configure::read('Conf.SAERP')) {
//            $this->set('types', $this->DataAuthorized->getAuthTypes(array('withSoustypes' => true, 'onlyActif' => false)));
            $this->set('types', $this->Session->read('Auth.TypesAll'));
        } else {
            $this->set('types', $this->Courrier->Soustype->Type->find('all', array('contain' => array('Soustype' => array('order' => 'Soustype.name ASC')), 'order' => array('Type.name ASC'), 'recursive' => -1)));
        }

//        $this->set('metadonnees', $this->DataAuthorized->getAuthMetadonnees(array('find' => 'list')));
        $this->set('metadonnees', $this->Session->read('Auth.MetasList'));

        $affairesuiviepars = $this->Courrier->Desktop->getDesktopsByService('all');
        ksort($affairesuiviepars);
        $this->set('affairesuiviepars', $affairesuiviepars);
        // Adresse
//        $this->set('bans', ClassRegistry::init('Ban')->find(
//                        'all', array(
//                    'order' => array('Ban.name ASC'),
//                    'contain' => array(
//                        'Bancommune' => array(
//                            'order' => 'Bancommune.name ASC'
//                        )
//                    )
//                        )
//                )
//        );
        $this->set('bans', $this->Session->read('Auth.Ban'));
        
        if( Configure::read('CD') == 81 ) {
            $this->set(
                'etat',
                array(
                    '-1' => 'Refusé',
                    '1' => 'En cours de traitement',
                    '2' => 'Clos'
                )
            );
        }
        else {
            $this->set(
                'etat',
                array(
                    '-1' => 'Refusé',
                    '0' => 'Validé',
                    '1' => 'En cours de traitement',
                    '2' => 'Clos'
                )
            );
        }
        $services = $this->Recherche->Service->find('list', array('order' => 'Service.name ASC'));
        /////////////////////////////
        $this->set('services', $services);
        if (Configure::read('Conf.SAERP')) {
            $marches = $this->Courrier->Marche->find(
                    'list', array(
                'joins' => array(
                    $this->Courrier->Marche->join('Operation')
                ),
                'conditions' => array(
                    'Marche.active' => true,
                    'Operation.active' => true
                ),
                'fields' => array(
                    'Marche.numero'
                ),
                'order' => array('Marche.name ASC')
                    )
            );
            $this->set(compact('marches'));
            $os = $this->Courrier->Ordreservice->find(
                    'list', array(
                'fields' => array(
                    'Ordreservice.name'
                ),
                'order' => array('Ordreservice.name ASC')
                    )
            );
//debug($os);
            $plis = $this->Courrier->Pliconsultatif->find(
                    'list', array(
                'fields' => array(
                    'Pliconsultatif.name'
                ),
                'order' => array('Pliconsultatif.name ASC')
                    )
            );
            $typedocument = array(
                'OS' => 'Ordre de service',
                'LN' => 'Lettre de notification',
                'LC' => 'Lettre de commandes',
                'PLI' => 'Pli'
            );
            $this->set(compact('typedocument'));
            $this->set(compact('os', 'plis'));

            $this->Organisme = ClassRegistry::init('Organisme');
            $this->set('activites', $this->Organisme->Activite->find('list', array('order' => array('Activite.name ASC'), 'conditions' => array('Activite.active' => true))));
            $this->set('events', $this->Organisme->Contact->Event->find('list', array('order' => array('Event.name ASC'), 'conditions' => array('Event.active' => true))));
            $this->set('fonctions', $this->Organisme->Contact->Fonction->find('list', array('order' => array('Fonction.name ASC'), 'conditions' => array('Fonction.active' => true))));
             
            $operations = $this->Courrier->Marche->Operation->find('list', array('conditions' => array('Operation.active' => true), 'order' => 'Operation.name ASC'));
            $this->set('operations', $operations);

            // Pour les N° RAR
            $rarId = $this->Courrier->Metadonnee->find('first', array('contain' => false, 'recursive' => -1, 'conditions' => array('Metadonnee.id' => Configure::read( 'MetadonnneRar.id') )));
            $rars = array();
            if( !empty($rarId) ) {
                $rars = $this->Courrier->CourrierMetadonnee->find(
                    'list',
                    array(
                        'fields' => array(
                            'CourrierMetadonnee.valeur'
                        ),
                        'conditions' => array(
                            'CourrierMetadonnee.metadonnee_id' => $rarId['Metadonnee']['id']
                        ),
                        'recursive' => -1,
                        'contain' => false
                    )
                );
            }
    //        debug($rars);
            $this->set('rars', $rars);
            $this->set('operations', $operations);
        }

        $conn = $this->Session->read('Auth.User.connName');
        $this->loadModel('Collectivite');
        $this->set('collectivite', $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $conn))));
        
    }

    /**
     * Index method
     *
     * @access public
     * @return void
     */
    public function index() {
        $this->set('ariane', array(__d('menu', 'Recherche', true)));
    }

    /**
     * Formulaire de recherche (enregistrement de recherche)
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @param integer $id
     * @param string $forcemode
     * @return void
     */
    public function formulaire($id = null, $forcemode = null) {
        //enregistrement des critères de la recherche

        if (!empty($this->request->data)) {
            $this->Jsonmsg->init();
            $data = array();
            $data['Recherche'] = $this->request->data['Recherche'];
            if (!empty($this->request->data['Contact']['Contact']) || empty($this->request->data['Recherche']['id'])) {
                $data['Contact'] = array();
                foreach ($this->request->data['Contact']['Contact'] as $kcontact => $contact) {
                    $data['Contact']['Contact'][$kcontact] = $contact;
                }
            } else {
                $query = "DELETE FROM recherches_contacts WHERE recherche_id = " . $this->request->data('Recherche.id') . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Type']['Type']) || empty($this->request->data['Recherche']['id'])) {
                foreach ($this->request->data['Type']['Type'] as $ktype => $type) {
                    $data['Type']['Type'][$ktype] = $type;
                }
            } else {
                $query = "DELETE FROM recherches_types WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Soustype']['Soustype']) || empty($this->request->data['Recherche']['id'])) {
                foreach ($this->request->data['Soustype']['Soustype'] as $ksoustype => $soustype) {
                    $data['Soustype']['Soustype'][$ksoustype] = $soustype;
                }
            } else {
                $query = "DELETE FROM recherches_soustypes WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Circuit']['Circuit']) || empty($this->request->data['Recherche']['id'])) {
                $data['Circuit'] = array();
                foreach ($this->request->data['Circuit']['Circuit'] as $kcircuit => $circuit) {
                    $data['Circuit']['Circuit'][$kcircuit] = $circuit;
                }
            } else {
                $query = "DELETE FROM recherches_circuits WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Dossier']['Dossier']) || empty($this->request->data['Recherche']['id'])) {
                $data['Dossier'] = array();
                foreach ($this->request->data['Dossier']['Dossier'] as $kdossier => $dossier) {
                    $data['Dossier']['Dossier'][$kdossier] = $dossier;
                }
            } else {
                $query = "DELETE FROM recherches_dossiers WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Affaire']['Affaire']) || empty($this->request->data['Recherche']['id'])) {
                $data['Affaire'] = array();
                foreach ($this->request->data['Affaire']['Affaire'] as $kaffaire => $affaire) {
                    $data['Affaire']['Affaire'][$kaffaire] = $affaire;
                }
            } else {
                $query = "DELETE FROM recherches_affaires WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            if (!empty($this->request->data['Tache']['Tache']) || empty($this->request->data['Recherche']['id'])) {
                $data['Tache'] = array();
                foreach ($this->request->data['Tache']['Tache'] as $ktache => $tache) {
                    $data['Tache']['Tache'][$ktache] = $tache;
                }
            } else {
                $query = "DELETE FROM recherches_taches WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            // Métadonnées
            if (!empty($this->request->data['Metadonnee']['Metadonnee']) || empty($this->request->data['Recherche']['id'])) {
                foreach ($this->request->data['Metadonnee']['Metadonnee'] as $kmetadonnee => $metadonnee) {
                    $data['Metadonnee']['Metadonnee'][$kmetadonnee] = $metadonnee;
                }
            } else {
                $query = "DELETE FROM recherches_metadonnees WHERE recherche_id = " . $this->request->data['Recherche']['id'] . ";";
                $this->Recherche->query($query);
            }
            // Valeur par Métadonnées
            if (!empty($this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'])) {
                foreach ($this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] as $typemetadonneeid => $valueMetadonnee) {
                    $data['Selectvaluemetadonnee']['Selectvaluemetadonnee'][$typemetadonneeid] = $valueMetadonnee;
                }
            }

            if (!empty($this->request->data['Desktop']['Desktop'])) {
                foreach ($this->request->data['Desktop']['Desktop'] as $kdesktop => $desktop) {
                    $data['Desktop']['Desktop'][$kdesktop] = $desktop;
                }
            }

            if (!empty($this->request->data['Organisme']['Organisme'])) {
                $data['Organisme'] = array();
                foreach ($this->request->data['Organisme']['Organisme'] as $korganisme => $organisme) {
                    $data['Organisme']['Organisme'][$korganisme] = $organisme;
                }
            }

            if (!empty($this->request->data['Service']['Service'])) {
                $data['Service'] = array();
                foreach ($this->request->data['Service']['Service'] as $kservice => $service) {
                    $data['Service']['Service'][$kservice] = $service;
                }
            }
            $data['Recherche']['user_id'] = $this->Session->read('Auth.User.id');
            $data['Recherche']['desktop_id'] = $this->Session->read('Auth.User.Desktop.id');

            $this->Recherche->begin();
            $success = $this->Recherche->saveAll($data);
            $metasList = $this->Recherche->Metadonnee->find(
                    'all', array(
                'recusive' => -1,
                'contain' => false
                    )
            );

            $typemetadonneeIds = Hash::extract($metasList, '{n}.Metadonnee.typemetadonnee_id');
            $valOfSelect = Configure::read('Selectvaluemetadonnee.id');

            // on regarde les anciens enregistrements entre le circut et les sous-types
            /* $records = $this->Recherche->RechercheSelectvaluemetadonnee->find(
              'list',
              array(
              'fields' => array(
              "RechercheSelectvaluemetadonnee.id",
              "RechercheSelectvaluemetadonnee.metadonnee_id"
              ),
              'conditions' => array(
              "RechercheSelectvaluemetadonnee.recherche_id" => $id
              )
              )
              );
              debug($records);

              $oldrecordsids = array_keys( $records );
              debug($oldrecordsids);
              $nouveauxids = Hash::filter( (array)Hash::extract( $data, 'Selectvaluemetadonnee.Selectvaluemetadonnee.{n}' ) );
              $idsenmoins = array_diff( $oldrecordsids, $nouveauxids );
              if( !empty( $idsenmoins ) && !empty( $id ) ) {
              $success = $this->Recherche->RechercheSelectvaluemetadonnee->deleteAll(
              array(
              "RechercheSelectvaluemetadonnee.recherche_id" => $id,
              "RechercheSelectvaluemetadonnee.metadonnee_id" => $idsenmoins
              )
              ) && $success;
              }
              debug($idsenmoins);

              // En plus -> Ajouter
              $idsenplus = array_diff( $nouveauxids, $oldrecordsids );
              debug($idsenplus);
              die();
              if( !empty( $idsenplus ) ) {
              foreach( $idsenplus as $idenplus ) {
              $record = array(
              'RechercheSelectvaluemetadonnee' => array(
              "recherche_id" => $id,
              "metadonnee_id" => $idenplus
              )
              );

              $this->Recherche->RechercheSelectvaluemetadonnee->create( $record );
              $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
              }
              } */

            // on regarde les nouveautés
//                    $nouveauxids = Hash::extract( $this->request->data, "Soustype.Soustype.{n}" );
            // si des différences existent on note les ids enlevés, les liens coupés entre circuit et sous-types
//                    $idsenmoins = array_diff( $oldrecordsids, $nouveauxids );

            foreach ($data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] as $meta_id => $selectValueMeta) {
                $metadon = $this->Recherche->Metadonnee->find('first', array('conditions' => array('Metadonnee.id' => $meta_id), 'recursive' => -1));

                if ($metadon['Metadonnee']['typemetadonnee_id'] != $valOfSelect) {
                    $metaValue = array(
                        'RechercheSelectvaluemetadonnee' => array(
                            'recherche_id' => $this->Recherche->id,
                            'selectvaluemetadonnee_id' => null,
                            'metadonnee_id' => $metadon['Metadonnee']['id'],
                            'valeur' => $selectValueMeta
                        )
                    );

                    $this->Recherche->RechercheSelectvaluemetadonnee->create($metaValue);
                    $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
                } else {
                    $metaValue = array(
                        'RechercheSelectvaluemetadonnee' => array(
                            'recherche_id' => $this->Recherche->id,
                            'selectvaluemetadonnee_id' => $selectValueMeta,
                            'metadonnee_id' => $metadon['Metadonnee']['id'],
                            'valeur' => null
                        )
                    );

                    // Comparaison des ids avant/apres pour mise à jour
                    // on regarde les anciens enregistrements entre la recherche et les valeurs de métadonnées
                    /* $records = $this->Recherche->RechercheSelectvaluemetadonnee->find(
                      'list',
                      array(
                      'fields' => array( "RechercheSelectvaluemetadonnee.id", "RechercheSelectvaluemetadonnee.selectvaluemetadonnee_id" ),
                      'conditions' => array(
                      "RechercheSelectvaluemetadonnee.recherche_id" => $id
                      )
                      )
                      ); */

                    if (empty($id)) {
                        $this->Recherche->RechercheSelectvaluemetadonnee->create($metaValue);
                        $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
                    }
                }
            }

            if ($success) {
                $this->Recherche->commit();
                $this->Jsonmsg->valid();
            } else {
                $this->Recherche->rollback();
            }
            $this->Jsonmsg->send();
        } else {
            $mode = "";
            if ($id != null) {
                if ($id == 0) {
                    $mode = "add";
                } else {
                    $mode = "edit";
                    if ($forcemode != null && $forcemode == 'search') {
                        $mode = 'search';
                        $this->set('recherches', $this->Recherche->find('list', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id')))));
                    }
                    $this->request->data = $this->Recherche->find(
                            'first', array(
                        'conditions' => array(
                            'Recherche.id' => $id
                        ),
                        'contain' => array(
                            'Desktop',
                            'Metadonnee',
                            'RechercheSelectvaluemetadonnee',
                            'Contact',
                            'Organisme',
                            'Type',
                            'Soustype',
                            'Dossier',
                            'Affaire',
                            'Tache',
                            'Circuit'
                        )
                            )
                    );
                    $metaToDisplay = array();
                    foreach ($this->request->data['RechercheSelectvaluemetadonnee'] as $i => $valueSelect) {
                        $key = $valueSelect['metadonnee_id'];
                        if ($valueSelect['selectvaluemetadonnee_id'] === null && $valueSelect['valeur'] === null) {
                            $metaToDisplay[$key] = null;
                        } else if ($valueSelect['selectvaluemetadonnee_id'] === null) {
                            $metaToDisplay[$key] = $valueSelect['valeur'];
                        } else {
                            $metaToDisplay[$key] = $valueSelect['selectvaluemetadonnee_id'];
                        }
                    }
                    $this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] = $metaToDisplay;
                }
            } else {
                $mode = "search";
                $this->set('recherches', $this->Recherche->find('list', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id')))));
            }
            $this->set('mode', $mode);
            $this->_setOptions();
        }
    }

    /**
     * Récupération de la liste des recherches
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function getRecherches() {

        $this->Recherche->recursive = -1;
        $recherches = $this->Recherche->find('all', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id'))));
        for ($i = 0; $i < count($recherches); $i++) {
            $recherches[$i]['right_view'] = true;
            $recherches[$i]['right_edit'] = true;
            $recherches[$i]['right_delete'] = true;
        }
        $this->set('recherches', $recherches);
    }

    /**
     * Suppression d'une recherche enregistrée
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant de la recherche enregistrée
     * @return void
     */
    public function delete($id = null) {
        $this->Jsonmsg->init(__d('default', 'delete.error'));
        $recherche = $this->Recherche->find('first', array('conditions' => array('Recherche.id' => $id)));
        $this->Recherche->begin();
        if ($this->Recherche->delete($id)) {
            $this->Recherche->commit();
            $this->Jsonmsg->valid(__d('default', 'delete.ok'));
        } else {
            $this->Recherche->rollback();
        }
        $this->Jsonmsg->send();
    }

    /**
     * Lancement de la recherche
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant de la recherche
     * @return void
     */
    public function recherche($id = null) {
//    $this->Recherche->recursive = -1;
        if ($id != null) {
            $this->request->data = $this->Recherche->find('first', array('conditions' => array('Recherche.id' => $id)));
        }
        $params = $this->request->data;

        if (!empty($params)) {
            
            foreach( array( 'cdate','cdatereception') as $criteresDatesDebut ){
                if( isset($this->request->data['Recherche'][$criteresDatesDebut])  && !empty($this->request->data['Recherche'][$criteresDatesDebut]) ) {
                    $dateReception = $this->request->data['Recherche'][$criteresDatesDebut];
                    if( strpos( $dateReception,  '/'  ) != 0 ) {
                        $debut = explode( '/', $dateReception );
                        $day = $debut[0];
                        $month = $debut[1];
                        $year = $debut[2];
                        $dateDebut = $year.'-'.$month.'-'.$day;
                        $this->request->data['Recherche'][$criteresDatesDebut] = $dateDebut;
                    }
                }
            }
            foreach( array( 'cdatefin','cdatereceptionfin') as $criteresDatesFin ){
                if( isset($this->request->data['Recherche'][$criteresDatesFin])  && !empty($this->request->data['Recherche'][$criteresDatesFin]) ) {
                    $dateReceptionFin = $this->request->data['Recherche'][$criteresDatesFin];
                    if( strpos( $dateReceptionFin,  '/'  ) != 0 ) {
                        $fin = explode( '/', $dateReceptionFin );
                        $day = $fin[0];
                        $month = $fin[1];
                        $year = $fin[2];
                        $dateFin = $year.'-'.$month.'-'.$day;
                        $this->request->data['Recherche'][$criteresDatesFin] = $dateFin;
                    }
                }
            }
            
//debug($this->request->data);
            if (isset($params['tri'])) {
                $tri = $params['tri'];
            }
            if (!empty($this->request->data['Recherche']['contactdept'])) {
                //Pour les régions
                $dept = ClassRegistry::init('Ban')->find(
                        'first', array(
                    'fields' => array('Ban.name', 'Ban.id'),
                    'conditions' => array(
                        'Ban.id' => $this->request->data['Recherche']['contactdept']
                    ),
                    'recursive' => -1,
                    'contain' => false
                        )
                );
//                $params['Recherche']['contactdept'] = $dept['Ban']['name'];
                $params['Recherche']['contactdept'] = $dept['Ban']['id'];
                // Pour les communes
                $commune = ClassRegistry::init('Ban')->Bancommune->find(
                        'first', array(
                    'fields' => array('Bancommune.name', 'Bancommune.id'),
                    'conditions' => array(
                        'Bancommune.id' => $this->request->data['Recherche']['contactcommune']
                    ),
                    'recursive' => -1,
                    'contain' => false
                        )
                );
//                $params['Recherche']['contactcommune'] = $commune['Bancommune']['name'];
                $params['Recherche']['contactcommune'] = $commune['Bancommune']['id'];
            }

            if (!Configure::read('Conf.SAERP')) {
//                $datasAuthorizedType = $this->DataAuthorized->getAuthTypes(array('withSoustypes' => true, 'onlyActif' => false));
                $datasAuthorizedType = $this->Session->read('Auth.TypesAll');
                foreach ($datasAuthorizedType as $t => $type) {
                    $sousTypeIds[$type['Type']['id']] = Hash::extract($type, 'Soustype.{n}.id');
                    $sousTypeIds['-1'] = array();
                }
            } else {
                $sousTypeIds = array();
            }
            //on lance la recherche
            $flux_tmp = array();
            $querydata = $this->Recherche->search($params, $id, $sousTypeIds);
            $this->Session->write('Recherche.Querydata', $querydata);            

            // On prépare les conditions en fonction des droits
            $flux = array();
            $this->loadModel('Desktop');
            $desktopId = $this->Session->read('Auth.User.Courrier.Desktop.id');
            $daros = $this->Desktop->getDaros($desktopId);
            $soustypes = $this->Recherche->Soustype->find('list', array('contain' => false));

            $allowedSoustypes = array();
            if (Configure::read('Use.AuthRights') && !Configure::read('Conf.SAERP')) {
//                $allowedSoustypes = $this->DataAuthorized->getAuthSoustypes(array('find' => 'all'));
                $allowedSoustypes = $this->Session->read('Auth.Soustypes');
            } else {
                $allowedSoustypes = $soustypes;
            }
//			foreach ($soustypes as $kSoustype => $soustype) {
//				$daco = $this->DataAcl->Daco->find('first', array('conditions' => array('Daco.model' => 'Soustype', 'Daco.foreign_key' => $kSoustype), 'contain' => false));
//				$chks = array();
//				foreach ($daros as $daro) {
//					$chks[] = $this->DataAcl->check($daro['Daro'], $daco['Daco']);
//				}
//				if (!in_array(false, $chks, true)) {
//					$allowedSoustypes[$kSoustype] = $soustype;
//				}
//			}
//debug( $this->DataAuthorized->getAuthSoustypes(array('find' => 'all')));
            // Si aucune condition, on ne retourne rien
            if (!empty($querydata)) {
                $querydata['limit'] = 10;
                if (isset($tri) && !empty($tri)) {
                    $querydata['order'] = $tri;
                } else {
                    $querydata['order'] = 'Courrier.datereception DESC';
                }
                // Aucune condition sur les utilisateurs du courrier si Admin
                $bureaux = Hash::extract($this->Session->read('Auth.User.SecondaryDesktops'), '{n}.Profil.name');
                if ($this->Session->read('Auth.User.Desktop.Profil.name') != 'Admin' && !in_array('Admin', $bureaux)) {
//                if( $this->Session->read('Auth.User.Desktop.Profil.name') != 'Admin' ) {
//                    $querydata['conditions'][] = array(
//                        'OR' => array(
//                            'OR' => array(
//                                array(
//                                    'Courrier.soustype_id' => array_keys($allowedSoustypes),
//                                    'Courrier.soustype_id IS NOT NULL'
//                                ),
//                                array(
//                                    'Courrier.soustype_id IS NULL'
//                                )
//                            ),
//                            'Soustype.id' => array_keys($allowedSoustypes),
//                            'Courrier.user_creator_id' => $this->Session->read('Auth.User.id'),
//                            'Courrier.desktop_creator_id' => $desktopId
//                        )
//                    );
                }

                $this->paginate = array('Courrier' => $querydata);

                $flux_tmp = $this->paginate('Courrier');
            }
//debug($querydata);
            foreach ($flux_tmp as $kflx => $flx) {
                $flx['parentInfo'] = $this->_dataParent($flx['Courrier']['parent_id']);
//				if (array_key_exists($flx['Soustype']['id'], $allowedSoustypes) || $flx['Courrier']['user_creator_id'] == $this->Session->read('Auth.User.id')) {
                $flx['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
                $flx['right_read'] = true;
                $flx['right_update'] = true;
//					$flx['right_delete'] = true;
                $flux[] = $flx;
//				}
            }
            if( Configure::read('CD') == 81 ) {
                $this->set(
                    'etat',
                    array(
                        '-1' => 'Refusé',
                        '1' => 'En cours de traitement',
                        '2' => 'Clos'
                    )
                );
            }
            else {
                $this->set(
                    'etat',
                    array(
                        '-1' => 'Refusé',
                        '0' => 'Validé',
                        '1' => 'En cours de traitement',
                        '2' => 'Clos'
                    )
                );
            }

            $this->set('flux', $flux);
            if (isset($tri) && $tri != '') {
                list($colone, $order) = explode(' ', $tri);

                switch ($colone) {
                    case 'Courrier.reference':
                        $ancien_name = 'reference';
                        break;
                    case 'Courrier.name':
                        $ancien_name = 'nom';
                        break;
                    case 'Courrier.objet':
                        $ancien_name = 'objet';
                        break;
                    case 'Organisme.name':
                        $ancien_name = 'expediteur';
                        break;
                    case 'Courrier.datereception':
                        $ancien_name = 'datereception';
                        break;
                    case 'Courrier.mail_retard_envoye':
                        $ancien_name = 'retard';
                        break;
                    default :
                        $ancien_name = '';
                        break;
                }
                $this->set('ancien_order', $order);
                $this->set('ancien_name', $ancien_name);
            }
        }
        
//        foreach( array( 'cdate','cdatereception') as $criteresDatesDebut ){
//            if( isset($this->request->data['Recherche'][$criteresDatesDebut])  && !empty($this->request->data['Recherche'][$criteresDatesDebut]) ) {
//                $dateReception = $this->request->data['Recherche'][$criteresDatesDebut];
//                $debut = explode( '-', $dateReception );
//                $day = $debut[0];
//                $month = $debut[1];
//                $year = $debut[2];
//                $dateDebut = $year.'/'.$month.'/'.$day;
//                $this->request->data['Recherche'][$criteresDatesDebut] = $dateDebut;
//            }
//        }
//        foreach( array( 'cdatefin','cdatereceptionfin') as $criteresDatesFin ){
//            if( isset($this->request->data['Recherche'][$criteresDatesFin])  && !empty($this->request->data['Recherche'][$criteresDatesFin]) ) {
//                $dateReceptionFin = $this->request->data['Recherche'][$criteresDatesFin];
//                $fin = explode( '-', $dateReceptionFin );
//                $day = $fin[0];
//                $month = $fin[1];
//                $year = $fin[2];
//                $dateFin = $year.'/'.$month.'/'.$day;
//                $this->request->data['Recherche'][$criteresDatesFin] = $dateFin;
//            }
//        }
    }

    /**
     * Export de la recherche au format CSV
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function exportcsv() {

        ini_set( 'max_execution_time', 0 );
        ini_set( 'memory_limit', '2048M' );
        if (!Configure::read('Conf.SAERP')) {
//            $datasAuthorizedType = $this->DataAuthorized->getAuthTypes(array('withSoustypes' => true, 'onlyActif' => false));
            $datasAuthorizedType = $this->Session->read('Auth.TypesAll');
            foreach ($datasAuthorizedType as $t => $type) {
                $sousTypeIds[$type['Type']['id']] = Hash::extract($type, 'Soustype.{n}.id');
            }
        } else {
            $sousTypeIds = array();
        }
//        $querydata = $this->Recherche->search(Hash::expand($this->request->params['named'], '__'), null, $sousTypeIds);
        $querydata = $this->Session->read('Recherche.Querydata');
        unset($querydata['limit']);
        $querydata['fields'][] = $this->Courrier->getVfMetaValues('metadonnees', Configure::read('Selectvaluemetadonnee.id'));
        $querydata['fields'][] = $this->Courrier->getVfMetaARenvoye('arenvoye', Configure::read('MetadonnneAREnvoye.id'));
        $querydata['order'] = 'Courrier.datereception ASC';

        if( Configure::read( 'CD' ) == 81 ) {
            $content = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
                <html>
                <head>
                    <title>Recherche</title>
                    <style type="text/css">
                        @page {
                            size: landscape;
                        }
                        .table {
                            width: 100%;
                            margin-bottom: 0!important;
                            border-bottom: 1px solid #ddd;
                            border-collapse: collapse!important;
                            border-radius: 1px;
                            font-size: 11px;
                        }
                        .table tr.entete td {
                            line-height: 1.42857143
                            vertical-align: top;
                            border-top: 1px solid #000;
                            border-bottom: 1px solid #000;
                            border-left: 1px solid #000;
                            padding: 8px!important;
                            font-weight: bold;
                        }
                        .table tr.corps td {
                            line-height: 1.42857143
                            vertical-align: top;
                            border-top: 1px solid #000;
                            border-bottom: 1px solid #000;
                            border-left: 1px solid #000;
                            padding: 8px!important;
                        }
                    </style>
                </head>
                <body>';      
//debug($flux);
//die();
//            $content.= '<page>';
            // Change the $content according to your requrement
            $content.= '<table class="table">';
//                        
            $content.= '<tr class="entete">
                <td>ID</td>
                <td>Statut</td>
                <td>Retard</td>
                <td>Référence</td>
                <td>Nom</td>
                <td>Objet</td>
                <td>Date entrée dans la collectivité</td>
                <td>Date de réception</td>
                <td>Direction concernée</td>
                <td>Service concerné</td>
                <td>Organisme</td>
                <td>Contact</td>
                <td>Type</td>
                <td>Sous-type</td>
                <td>Dossier</td>
                <td>Affaire</td>
                <td>Nom de la tâche associée</td>
                <td>Méta-données</td>
            </tr>';
        }

        $flux = $this->Courrier->find('all', $querydata);
        foreach ($flux as $i => $flx) {

            /*
            $flux[$i]['Courrier'] = $flx['Courrier'];
            */

            if( Configure::read('Conf.SAERP' ) ) {
                if (isset($flx['Metadonnee']['name']) && $flx['Metadonnee']['id'] == Configure::read('MetadonnneMontant.id')) {
                    $flux[$i]['Lettre']['montant'] = $flx['CourrierMetadonnee']['valeur'];
                } else {
                    $flux[$i]['Lettre']['montant'] = '';
                }
            }

            if( !empty( $flux[$i]['Bancontenu']['bannette_id'] ) && $flux[$i]['Bancontenu']['bannette_id'] == 4 && $flux[$i]['Bancontenu']['etat'] == -1 ) {
                $flux[$i]['Courrier']['daterefus'] = $flux[$i]['Bancontenu']['modified'];
            }
            else {
                $flux[$i]['Courrier']['daterefus'] = '';
            }

            if( !empty( $flux[$i]['Bancontenu']['bannette_id'] ) && $flux[$i]['Bancontenu']['bannette_id'] == 4 && $flux[$i]['Bancontenu']['etat'] == 1 ) {
                $flux[$i]['Courrier']['datevalidation'] = $flux[$i]['Bancontenu']['modified'];
            }
            else {
                $flux[$i]['Courrier']['datevalidation'] = '';
            }

            if( !empty( $flux[$i]['Bancontenu']['bannette_id'] ) && $flux[$i]['Bancontenu']['bannette_id'] == 4 && $flux[$i]['Bancontenu']['etat'] == 2 ) {
                $flux[$i]['Courrier']['datecloture'] = $flux[$i]['Bancontenu']['modified'];
            }
            else {
                $flux[$i]['Courrier']['datecloture'] = '';
            }
 
            if( Configure::read( 'CD' ) == 81 ) {
                ////// pour e cD81 ///////////////
                $etat=  array(
                    '-1' => 'Refusé',
                    '0' => 'Traité à clore',
                    '1' => 'En cours de traitement',
                    '2' => 'Clos'
                );
                $services =  $this->Courrier->Service->find('list');
                $datasPassed = Hash::expand( $this->request->params['named'], '__' );
                if( !empty($datasPassed['Recherche']['cretard']) ) {
                    $retard = $datasPassed['Recherche']['cretard'];
                }
                else {
                    $retard = ( Hash::get( $flx, 'Courrier.mail_retard_envoye' ) ? 'Oui' : 'Non');
                }
                $statut = Hash::get( $etat, Hash::get( $flx, 'Bancontenu.etat' ) );
                $objet =  $flx['Courrier']['objet'];
                if( strpos( $objet, '<!DOCTYPE') !== false || strpos( $objet, '<html>') !== false || strpos( $objet, '--_000') !== false ) {
                    $objet = strip_tags( $objet, "<html>" );
                    $substring = substr($objet ,strpos($objet ,"<style"),strpos($objet ,"</style>"));
                    $objet = str_replace($substring,"",$objet);
                    $objet = trim($objet);
                }

                $content.= '<tr class="corps">
                    <td>'.$flx['Courrier']['id'].'</td>
                    <td>'.$statut.'</td>
                    <td>'.$retard.'</td>
                    <td>'.$flx['Courrier']['reference'].'</td>
                    <td>'.$flx['Courrier']['name'].'</td>
                    <td>'.$objet.'</td>
                    <td>'. strftime('%d/%m/%Y', strtotime(Hash::get( $flx, 'Courrier.date' ))).'</td>
                    <td>'.strftime('%d/%m/%Y', strtotime(Hash::get( $flx, 'Courrier.datereception' ) ) ).'</td>
                    <td>'.Hash::get( $flx, 'Service.name' ).'</td>
                    <td>'.Hash::get( $services, Hash::get( $flx, 'Service.parent_id' ) ).'</td>
                    <td>'.Hash::get( $flx, 'Organisme.name' ).'</td>
                    <td>'.Hash::get( $flx, 'Contact.name' ).'</td>
                    <td>'.Hash::get( $flx, 'Type.name' ).'</td>
                    <td>'.Hash::get( $flx, 'Soustype.name' ).'</td>
                    <td>'.Hash::get( $flx, 'Dossier.name' ).'</td>
                    <td>'.Hash::get( $flx, 'Affaire.name' ).'</td>
                    <td>'.Hash::get( $flx, 'Tache.name' ).'</td>
                    <td>'.Hash::get( $flx, 'Metadonnee.valeur' ).'</td>
                </tr>';
            }
        }
        if( Configure::read( 'CD' ) == 81 ) {
            $content.= '</table>';
            $content.='</body></html>';
    //            echo $content;

            $generatedFile = file_put_contents(TMP . DS . 'resultats_recherche_'.date( 'Ymd-His' ).'.html', utf8_decode($content) );
            $document_content = file_get_contents(TMP . DS . 'resultats_recherche_'.date( 'Ymd-His' ).'.html');

    //            $document_content = $this->Conversion->convertirFlux($document_content, 'html', 'pdf');
            Configure::write('debug', 0);
            header("Content-type: text/html");
            header("Content-Disposition: attachment; filename=\"resultats_recherche_".date( 'Ymd-His' ).".html\""); // use 'attachment' to force a file download

            echo $document_content;

            $pathToUnlink = TMP . DS . "resultats_recherche_".date( 'Ymd-His' ).".html";
            if( file_exists( $pathToUnlink ) ) {
                unlink($pathToUnlink);
            }
            exit();
        }
        else {
            $this->layout = '';
        }
    
        $this->set(
            'etat',
            array(
                '-1' => 'Refusé',
                '0' => 'Traité à clore',
                '1' => 'En cours de traitement',
                '2' => 'Clos'
            )
        );
        $this->set('services', $this->Session->read('Auth.Services') );
        $this->set(compact('flux'));
    }

    public function charge_soustypes() {
//        $this->autoRender = false;
        $resultats = array();
        foreach ($this->request->data['TypeChoix'] as $key => $value) {
//            if ($key != 0) {
            $soustypes = $this->Recherche->Soustype->find('list', array('fields' => array('Soustype.id', 'Soustype.name'), 'conditions' => array('Soustype.type_id' => $value)));
            foreach ($soustypes as $key => $value) {
                $resultats[$key] = $value;
            }
//            }
        }
//debug($resultats);
        $this->set('resultats', $resultats);
    }

    /**
     * Export des données pour la SAERP
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function export() {
        
        ini_set( 'max_execution_time', 0 );
        ini_set( 'memory_limit', '2048M' );
        $this->set('ariane', array(
            '<a href="/environnement/userenv/0">' . __d('menu', 'Recherche') . '</a>',
            __d('menu', 'export', true)
        ));
        if( !empty($this->request->data) ) {
            if (Configure::read('Conf.SAERP')) {
                if( !empty($this->request->data['Recherche']['cdatefilter']) ) {
                    $dateSaisie = $this->request->data['Recherche']['cdatefilter'];
                    $debut = explode( '/', $dateSaisie );
                    $day = $debut[0];
                    $month = $debut[1];
                    $year = $debut[2];
                    $dateDebut = $year.'-'.$month.'-'.$day;
                    $this->request->data['Recherche']['cdatefilter'] = $dateDebut;
                }
                if( !empty($this->request->data['Recherche']['cdatefilterfin']) ) {
                    $dateSaisie = $this->request->data['Recherche']['cdatefilterfin'];
                    $fin = explode( '/', $dateSaisie );
                    $day = $fin[0];
                    $month = $fin[1];
                    $year = $fin[2];
                    $dateFin = $year.'-'.$month.'-'.$day;
                    $this->request->data['Recherche']['cdatefilterfin'] = $dateFin;
                }
            }

            if( $this->request->data['Recherche']['ctypedocument'] == 'PLI' ) {
                $queryDataPli = $this->Consultation->search($this->request->data);
                $queryDataPli['order'] = 'Operation.name ASC';
                $queryDataPli['group'] = array( 'Operation.name', 'Consultation.name', 'Pliconsultatif.id', 'Origineflux.id', 'Courrier.id', 'Consultation.id', 'Operation.id', 'CourrierMetadonnee.id', 'Metadonnee.id', 'Organisme.id', 'Contact.id' );
                $queryDataPli['limit'] = 1000;
                $queryDataPli['maxLimit'] = 10000;
                $this->paginate = $queryDataPli;
                $flux_tmp = $this->paginate($this->Consultation);
                $flux = array();
                foreach( $flux_tmp as $key => $flx) {
                    $flux[$flx['Consultation']['id']] = $flx;
                }
            }
            else if( $this->request->data['Recherche']['ctypedocument'] == 'OS' ) {
                $queryDataOS = $this->Ordreservice->search($this->request->data);
                $queryDataOS['order'] = array( 'Operation.name ASC', 'Marche.numero ASC' );
                $queryDataOS['group'] = array( 'Operation.name', 'Marche.numero', 'Ordreservice.id', 'Courrier.id', 'Marche.id', 'Operation.id', 'CourrierMetadonnee.id', 'Metadonnee.id', 'Organisme.id', 'Contact.id'  );
                $queryDataOS['limit'] = 20;
                $queryDataOS['maxLimit'] = 10000;
                $this->paginate = $queryDataOS;
                $flux = $this->paginate($this->Ordreservice);
            }
            else {
                $querydata = $this->Recherche->search( $this->request->data, null, array() );
                $querydata['order'] = 'Operation.name ASC';
                $this->paginate = array( 'Courrier' => $querydata );
                $flux = $this->paginate( 'Courrier' );
            }
//debug($flux);
            foreach ($flux as $i => $flx) {

                $associations = $this->Courrier->Soustype->Type->Operation->Intituleagentbyoperation->find(
                        'all', array(
                    'conditions' => array(
                        'Intituleagentbyoperation.operation_id' => $flx['Operation']['id']
                    ),
                    'contain' => array(
                        'Desktopmanager' => array(
                            'Desktop' => array(
                                'User',
                                'SecondaryUser'
                            )
                        ),
                        'Intituleagent' => array(
                            'conditions' => array(
                                'Intituleagent.name' => array('RO', 'AO', 'GEST')
                            ),
                            'order' => array('Intituleagent.name ASC')
                        )
                    )
                        )
                );

                if (!empty($associations[0]['Desktopmanager']['Desktop'][0]['User'])) {
                    $flux[$i]['Operation']['ao'] = @$associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] . ' ' . @$associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['nom'];
                    $flux[$i]['Operation']['gest'] = @$associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] . ' ' . @$associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['nom'];
                    $flux[$i]['Operation']['ro'] = @$associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] . ' ' . @$associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['nom'];
                } else {
                    $flux[$i]['Operation']['ao'] = @$associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] . ' ' . @$associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'];
                    $flux[$i]['Operation']['gest'] = @$associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] . ' ' . @$associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'];
                    $flux[$i]['Operation']['ro'] = @$associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] . ' ' . @$associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'];
                }
                if (isset($flx['Metadonnee']['name']) && $flx['Metadonnee']['id'] == Configure::read('MetadonnneMontant.id')) {
                    $flux[$i]['Lettre']['montant'] = $flx['CourrierMetadonnee']['valeur'];
                }
                $flx['Courrier']['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
                
                // destinataire du flux
//                $civilite = array(
//                    'Monsieur' => 'Monsieur',
//                    'Madame' => 'Madame', 
//                    'Monsieur et Madame' => 'Monsieur et Madame'
//                );
//                $flx['Destinataire']['name'] = @$flx['Organisme']['name']. ' / '.$civilite[$flx['Contact']['civilite']].' '.$flx['Contact']['name'];
//                
//                $flux[$i]['Destinataire']['name'] = $flx['Destinataire']['name'];
//                $flux[$i]['Destinataire']['numvoie'] = $flx['Contact']['numvoie'];
//                $flux[$i]['Destinataire']['nomvoie'] = $flx['Contact']['nomvoie'];
//                $flux[$i]['Destinataire']['compl'] = $flx['Contact']['compl'];
//                $flux[$i]['Destinataire']['cp'] = $flx['Contact']['cp'];
//                $flux[$i]['Destinataire']['ville'] = $flx['Contact']['ville'];
                    
                $flux[$i]['Courrier'] = $flx['Courrier'];
                $flux[$i]['right_read'] = true;
                $flux[$i]['right_view'] = true;
            }

            $countFlux = count($flux);
            $this->set(compact('flux', 'countFlux'));
        }
        
        if( !empty($this->request->data['Recherche']['cdatefilter']) ) {
            $dateSaisie = $this->request->data['Recherche']['cdatefilter'];
            $debut = explode( '-', $dateSaisie );
            $day = $debut[0];
            $month = $debut[1];
            $year = $debut[2];
            $dateDebut = $year.'/'.$month.'/'.$day;
            $this->request->data['Recherche']['cdatefilter'] = $dateDebut;
        }
        if( !empty($this->request->data['Recherche']['cdatefilterfin']) ) {
            $dateSaisie = $this->request->data['Recherche']['cdatefilterfin'];
            $fin = explode( '-', $dateSaisie );
            $day = $fin[0];
            $month = $fin[1];
            $year = $fin[2];
            $dateFin = $year.'/'.$month.'/'.$day;
            $this->request->data['Recherche']['cdatefilterfin'] = $dateFin;
        }

        $this->_setOptions();
    }

    /**
     * Export des LN pour la SAERP
     *
     * @logical-group Recherches
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function exportLettre() {
        
        ini_set( 'max_execution_time', 0 );
        ini_set( 'memory_limit', '2048M' );
        $this->set('ariane', array(
            '<a href="/environnement/userenv/0">' . __d('menu', 'Recherche') . '</a>',
            __d('menu', 'export', true)
        ));

        if (isset($this->request->params['named']['Recherche__ctypedocument']) && $this->request->params['named']['Recherche__ctypedocument'] == 'OS') {
            $querydata = $this->Ordreservice->search(Hash::expand($this->request->params['named'], '__'));
            unset($querydata['limit']);
            $querydata['order'] = array( 'Operation.name ASC', 'Marche.name ASC' );
            $flux = $this->Ordreservice->find('all', $querydata);   
        } else if (isset($this->request->params['named']['Recherche__ctypedocument']) && $this->request->params['named']['Recherche__ctypedocument'] == 'PLI') {
//            $querydata = $this->Pliconsultatif->search(Hash::expand($this->request->params['named'], '__'));
            $querydata = $this->Consultation->search(Hash::expand($this->request->params['named'], '__'));
            unset($querydata['limit']);
            $querydata['order'] = 'Operation.name ASC';
            $flux_tmp = $this->Consultation->find( 'all', $querydata );
            $flux = array();
            foreach( $flux_tmp as $key => $flx) {
                $flux[$flx['Consultation']['id']] = $flx;
            }

        } else {
            $querydata = $this->Recherche->search(Hash::expand($this->request->params['named'], '__'), null, array());
            unset($querydata['limit']);
            $querydata['order'] = 'Operation.name ASC';
            $flux = $this->Courrier->find('all', $querydata);
        }
        foreach ($flux as $i => $flx) {
             // Données méta-données
            $courriers_metadonnees = $this->Courrier->CourrierMetadonnee->find(
                'all',
                array(
                    'conditions' => array(
                        'CourrierMetadonnee.courrier_id' => $flx['Courrier']['id']
                    ),
                    'contain' => array(
                        'Metadonnee' => array(
                            'order' => array(
                                'Metadonnee.name ASC'
                            ),
                            'Selectvaluemetadonnee'
                        )
                    )
                )
            );
//debug($courriers_metadonnees);
//die();
            $metas= array();
            foreach( $courriers_metadonnees as $c => $valeurs) {
                foreach( $valeurs['Metadonnee']['Selectvaluemetadonnee'] as $s => $valeurChoisi ) {
                    $selected[$valeurChoisi['id']] = $valeurChoisi['name'];
                }
                if( $valeurs['Metadonnee']['typemetadonnee_id'] == 4) {
                    $metas[$c] = $valeurs['Metadonnee']['name']. " : ".$selected[$valeurs['CourrierMetadonnee']['valeur']];
                }
                else {
                    $metas[$c] = $valeurs['Metadonnee']['name']. " : ".$valeurs['CourrierMetadonnee']['valeur'];
                }
            }
            $flux[$i]['Metadonnee']['valeur'] = implode(' // ', $metas);
            

            $associations = $this->Courrier->Soustype->Type->Operation->Intituleagentbyoperation->find(
                    'all', array(
                'conditions' => array(
                    'Intituleagentbyoperation.operation_id' => $flx['Operation']['id']
                ),
                'contain' => array(
                    'Desktopmanager' => array(
                        'Desktop' => array(
                            'User',
                            'SecondaryUser'
                        )
                    ),
                    'Intituleagent' => array(
                        'conditions' => array(
                            'Intituleagent.name' => array('RO', 'RO m', 'AO', 'GEST')
                        ),
                        'order' => array('Intituleagent.name ASC')
                    )
                )
                    )
            ); 
            if (!empty($associations[0]['Desktopmanager']['Desktop'][0]['User'])) {
                $flux[$i]['Operation']['ao'] = isset( $associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] ) ? ($associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['nom'] ) : ($associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom']);
                $flux[$i]['Operation']['gest'] = isset( $associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] ) ? ($associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['nom'] ) : ($associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom']);
                $flux[$i]['Operation']['ro'] = isset( $associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'] ) ? ($associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['nom'] ) : ($associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom']);
            } else {
                $flux[$i]['Operation']['ao'] = isset( $associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] ) ? ($associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[0]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'] ) : ($associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[0]['Desktopmanager']['Desktop'][0]['User'][0]['nom']);
                $flux[$i]['Operation']['gest'] = isset( $associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] ) ? ($associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[1]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'] ) : ($associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[1]['Desktopmanager']['Desktop'][0]['User'][0]['nom']);
                $flux[$i]['Operation']['ro'] = isset( $associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'] ) ? ($associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['prenom'].' '.@$associations[2]['Desktopmanager']['Desktop'][0]['SecondaryUser'][0]['nom'] ) : ($associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['prenom'].' '.@$associations[2]['Desktopmanager']['Desktop'][0]['User'][0]['nom']);
            } 
            if (isset($flx['Metadonnee']['name']) && $flx['Metadonnee']['id'] == Configure::read('MetadonnneMontant.id')) {
                $flux[$i]['Lettre']['montant'] = $flx['CourrierMetadonnee']['valeur'];
            } else {
                $flux[$i]['Lettre']['montant'] = '';
            }
            $flx['Courrier']['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
            
             // destinataire du flux
            $civilite = array(
                '1' => 'Monsieur',
                '2' => 'Madame',
                '3' => 'Monsieur et Madame',
                '' => ''
            );            
            if(Configure::read('CD') == 81) {
                $civilite = array(
                    '1' => 'Monsieur',
                    '2' => 'Madame',
                    '3' => 'Madame et Monsieur',
                    '4' => 'Messieurs',
                    '5' => 'Mesdames',
                    '6' => "Mesdames et Messieurs",
                    '-1' => "--Sans--"
                );
            }
            
            $flx['Destinataire']['name'] = @$flx['Organisme']['name']. ' / '.@$civilite[$flx['Contact']['civilite']].' '.@$flx['Contact']['name'];

            $flux[$i]['Destinataire']['name'] = $flx['Destinataire']['name'];
            $flux[$i]['Destinataire']['siret'] = $flx['Organisme']['siret'];
            $flux[$i]['Destinataire']['numvoie'] = $flx['Contact']['numvoie'];
            $flux[$i]['Destinataire']['nomvoie'] = $flx['Contact']['nomvoie'];
            $flux[$i]['Destinataire']['compl'] = $flx['Contact']['compl'];
            $flux[$i]['Destinataire']['cp'] = $flx['Contact']['cp'];
            $flux[$i]['Destinataire']['ville'] = $flx['Contact']['ville'];
            
            $flux[$i]['Courrier'] = $flx['Courrier'];
        }

//debug($flux);
//die();

        $this->set(compact('flux'));
        $this->layout = '';

        $this->_setOptions();
    }

    
    
    /**
     *  Fonction permettant de récupérer les données parentes de flux réponses
     *
     */
    protected function _dataParent($courrierParent_id) {
        $this->Bancontenu = ClassRegistry::init('Bancontenu');
        $courrierParent = $this->Bancontenu->Courrier->find(
                'first', array(
            'fields' => array_merge(
                    $this->Bancontenu->Courrier->fields(), $this->Bancontenu->Courrier->Organisme->fields(), $this->Bancontenu->Courrier->Contact->fields(), $this->Bancontenu->Courrier->Soustype->fields(), $this->Bancontenu->Courrier->Soustype->Type->fields(), $this->Bancontenu->Courrier->Desktop->fields()
            ),
            'conditions' => array(
                'Courrier.id' => $courrierParent_id
            ),
            'joins' => array(
                $this->Bancontenu->Courrier->join('Organisme'),
                $this->Bancontenu->Courrier->join('Contact'),
                $this->Bancontenu->Courrier->join('Soustype'),
                $this->Bancontenu->Courrier->Soustype->join('Type'),
                $this->Bancontenu->Courrier->join('Desktop')
            ),
            'recursive' => -1
                )
        );
//debug($courrierParent);
        return $courrierParent;
    }

}

?>
